﻿using System;
using System.Reflection;
using System.Threading;
using System.Windows;
using System.Windows.Interactivity;

namespace MetroWindow.Interactivity
{
    /// <summary>
    /// Sets the designated property to the supplied value. TargetObject
    /// optionally designates the object on which to set the property. If
    /// TargetObject is not supplied then the property is set on the object
    /// to which the trigger is attached.
    /// </summary>
    public class SetPropertyDelayedAction : TriggerAction<FrameworkElement>
    {
        /// <summary>
        /// The delay the property will be executed in response to the trigger.
        /// </summary>
        public int Delay
        {
            get { return (int)GetValue(DelayProperty); }
            set { SetValue(DelayProperty, value); }
        }

        public static readonly DependencyProperty DelayProperty
            = DependencyProperty.Register("Delay", typeof(int), typeof(SetPropertyDelayedAction));

        /// <summary>
        /// The value property will be checked against before being set.
        /// </summary>
        public object CheckPropertyValue
        {
            get { return GetValue(CheckPropertyValueProperty); }
            set { SetValue(CheckPropertyValueProperty, value); }
        }

        public static readonly DependencyProperty CheckPropertyValueProperty
            = DependencyProperty.Register("CheckPropertyValue", typeof(object), typeof(SetPropertyDelayedAction));

        /// <summary>
        /// The property to be executed in response to the trigger.
        /// </summary>
        public string PropertyName
        {
            get { return (string)GetValue(PropertyNameProperty); }
            set { SetValue(PropertyNameProperty, value); }
        }

        public static readonly DependencyProperty PropertyNameProperty
            = DependencyProperty.Register("PropertyName", typeof(string), typeof(SetPropertyDelayedAction));


        /// <summary>
        /// The value to set the property to.
        /// </summary>
        public object PropertyValue
        {
            get { return GetValue(PropertyValueProperty); }
            set { SetValue(PropertyValueProperty, value); }
        }

        public static readonly DependencyProperty PropertyValueProperty
            = DependencyProperty.Register("PropertyValue", typeof(object), typeof(SetPropertyDelayedAction));


        /// <summary>
        /// Specifies the object upon which to set the property.
        /// </summary>
        public object TargetObject
        {
            get { return GetValue(TargetObjectProperty); }
            set { SetValue(TargetObjectProperty, value); }
        }

        public static readonly DependencyProperty TargetObjectProperty
            = DependencyProperty.Register("TargetObject", typeof(object), typeof(SetPropertyDelayedAction));

        /// <summary>
        /// 
        /// </summary>
        /// <param name="parameter"></param>
        protected override void Invoke(object parameter)
        {
            object target = this.TargetObject ?? this.AssociatedObject;

            PropertyInfo propertyInfo = target.GetType()
                                              .GetProperty(this.PropertyName,
                                                           BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.InvokeMethod);

            Int32 delay = this.Delay;

            ThreadPool.QueueUserWorkItem(new WaitCallback((si) =>
            {
                Thread.Sleep(delay);

                Application.Current.Dispatcher.BeginInvoke(new Action(() =>
                {
                    Boolean doTheJob = true;

                    if (this.CheckPropertyValue != null)
                        doTheJob = this.CheckPropertyValue.Equals(true);

                    if(doTheJob)
                        propertyInfo.SetValue(target, this.PropertyValue, null);

                }));
            }));
        }
    }
}